@@ -1,6 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import with_statement
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -10,15 +10,14 @@ import sys, os
 
 from calibre.customize.conversion import OutputFormatPlugin
 from calibre.customize.conversion import OptionRecommendation
-from polyglot.builtins import unicode_type
 
 
-class LRFOptions(object):
+class LRFOptions:
 
     def __init__(self, output, opts, oeb):
         def f2s(f):
             try:
-                return unicode_type(f[0])
+                return str(f[0])
             except:
                 return ''
         m = oeb.metadata
@@ -32,13 +31,13 @@ class LRFOptions(object):
         self.title_sort = self.author_sort = ''
         for x in m.creator:
             if x.role == 'aut':
-                self.author = unicode_type(x)
-                fa = unicode_type(getattr(x, 'file_as', ''))
+                self.author = str(x)
+                fa = str(getattr(x, 'file_as', ''))
                 if fa:
                     self.author_sort = fa
         for x in m.title:
-            if unicode_type(x.file_as):
-                self.title_sort = unicode_type(x.file_as)
+            if str(x.file_as):
+                self.title_sort = str(x.file_as)
         self.freetext = f2s(m.description)
         self.category = f2s(m.subject)
         self.cover = None
@@ -117,8 +116,7 @@ class LRFOutput(OutputFormatPlugin):
         ),
         OptionRecommendation(name='render_tables_as_images',
             recommended_value=False,
-            help=_('Render tables in the HTML as images (useful if the '
-                'document has large or complex tables)')
+            help=_('This option has no effect')
         ),
         OptionRecommendation(name='text_size_multiplier_for_rendered_tables',
             recommended_value=1.0,
@@ -188,7 +186,7 @@ class LRFOutput(OutputFormatPlugin):
         self.flatten_toc()
 
         from calibre.ptempfile import TemporaryDirectory
-        with TemporaryDirectory(u'_lrf_output') as tdir:
+        with TemporaryDirectory('_lrf_output') as tdir:
             from calibre.customize.ui import plugin_for_output_format
             oeb_output = plugin_for_output_format('oeb')
             oeb_output.convert(oeb, tdir, input_plugin, opts, log)
