@@ -14,7 +14,7 @@ import glob, os
 from calibre.customize.conversion import (OutputFormatPlugin,
     OptionRecommendation)
 from calibre.ptempfile import TemporaryDirectory
-from polyglot.builtins import iteritems, unicode_type
+from polyglot.builtins import iteritems
 
 UNITS = ('millimeter', 'centimeter', 'point', 'inch' , 'pica' , 'didot',
         'cicero', 'devicepixel')
@@ -148,13 +148,13 @@ class PDFOutput(OutputFormatPlugin):
         # Ensure Qt is setup to be used with WebEngine
         # specialize_options is called early enough in the pipeline
         # that hopefully no Qt application has been constructed as yet
-        from PyQt5.QtWebEngineCore import QWebEngineUrlScheme
-        from PyQt5.QtWebEngineWidgets import QWebEnginePage  # noqa
+        from qt.webengine import QWebEngineUrlScheme
+        from qt.webengine import QWebEnginePage  # noqa
         from calibre.gui2 import must_use_qt
         from calibre.constants import FAKE_PROTOCOL
         scheme = QWebEngineUrlScheme(FAKE_PROTOCOL.encode('ascii'))
         scheme.setSyntax(QWebEngineUrlScheme.Syntax.Host)
-        scheme.setFlags(QWebEngineUrlScheme.SecureScheme)
+        scheme.setFlags(QWebEngineUrlScheme.Flag.SecureScheme)
         QWebEngineUrlScheme.registerScheme(scheme)
         must_use_qt()
         self.input_fmt = input_fmt
@@ -193,10 +193,11 @@ class PDFOutput(OutputFormatPlugin):
 
     def get_cover_data(self):
         oeb = self.oeb
-        if (oeb.metadata.cover and unicode_type(oeb.metadata.cover[0]) in oeb.manifest.ids):
-            cover_id = unicode_type(oeb.metadata.cover[0])
+        if (oeb.metadata.cover and str(oeb.metadata.cover[0]) in oeb.manifest.ids):
+            cover_id = str(oeb.metadata.cover[0])
             item = oeb.manifest.ids[cover_id]
-            self.cover_data = item.data
+            if isinstance(item.data, bytes):
+                self.cover_data = item.data
 
     def process_fonts(self):
         ''' Make sure all fonts are embeddable '''
