@@ -307,7 +307,7 @@ class Worker(Thread):  # Get details {{{
                 text() = "Buscar productos similares por categoría" or
                 text() = "Ricerca articoli simili per categoria" or
                 text() = "Rechercher des articles similaires par rubrique" or
-                text() = "Procure por itens similares por categoria" or
+                text() = "Procure por items similares por categoria" or
                 text() = "関連商品を探す"
             ]/../descendant::ul/li
         '''
@@ -316,7 +316,7 @@ class Worker(Thread):  # Get details {{{
             r'([0-9.,]+) ?(out of|von|van|su|étoiles sur|つ星のうち|de un máximo de|de|av) '
             r'([\d\.]+)( (stars|Sternen|stelle|estrellas|estrelas|sterren|stjärnor)){0,1}'
         )
-        self.ratings_pat_cn = re.compile('平均([0-9.]+)')
+        self.ratings_pat_cn = re.compile('([0-9.]+) 颗星，最多 5 颗星')
         self.ratings_pat_jp = re.compile(r'\d+つ星のうち([\d\.]+)')
 
         lm = {
@@ -508,8 +508,12 @@ class Worker(Thread):  # Get details {{{
         if self.filter_result(mi, self.log):
             self.result_queue.put(mi)
 
-    def totext(self, elem):
-        return self.tostring(elem, encoding='unicode', method='text').strip()
+    def totext(self, elem, only_printable=False):
+        res = self.tostring(elem, encoding='unicode', method='text')
+        if only_printable:
+            filtered_characters = list(s for s in res if s.isprintable())
+            res = ''.join(filtered_characters).strip()
+        return res
 
     def parse_title(self, root):
 
@@ -672,6 +676,10 @@ class Worker(Thread):  # Get details {{{
             desc = root.xpath('//div[@id="ps-content"]/div[@class="content"]')
             if desc:
                 ans = self._render_comments(desc[0])
+            else:
+                ns = tuple(self.selector('#bookDescription_feature_div .a-expander-content'))
+                if ns:
+                    ans = self._render_comments(ns[0])
 
         desc = root.xpath(
             '//div[@id="productDescription"]/*[@class="content"]')
@@ -720,21 +728,14 @@ class Worker(Thread):  # Get details {{{
                         if series:
                             ans = (series, series_index)
         else:
-            series = root.xpath('//div[@id="seriesBullet_feature_div"]')
+            series = root.xpath('//div[@id="seriesBulletWidget_feature_div"]')
             if series:
-                series = series[0]
-                spans = series.xpath('descendant::span')
-                if spans:
-                    span = spans[0]
-                    b = span.xpath('./b')
-                    a = span.xpath('./a')
-                    if a and b:
-                        series = self.tostring(a[0], encoding='unicode', method='text', with_tail=False).strip()
-                        if series:
-                            raw = self.tostring(b[0], encoding='unicode', method='text', with_tail=False).strip()
-                            m = re.search(r'[0-9.]+', raw)
-                            if m is not None:
-                                ans = (series, float(m.group()))
+                a = series[0].xpath('descendant::a')
+                if a:
+                    raw = self.tostring(a[0], encoding='unicode', method='text', with_tail=False)
+                    m = re.search(r'(?:Book|Libro)\s+(?P<index>[0-9.]+)\s+(?:of|de)\s+([0-9.]+)\s*:\s*(?P<series>.+)', raw.strip())
+                    if m is not None:
+                        ans = (m.group('series').strip(), float(m.group('index')))
 
         # This is found on Kindle edition pages on amazon.com
         if ans == (None, None):
@@ -883,7 +884,7 @@ class Worker(Thread):  # Get details {{{
                 self.parse_detail_cells(mi, cells[0], cells[1])
 
     def parse_detail_cells(self, mi, c1, c2):
-        name = self.totext(c1).strip().strip(':').strip()
+        name = self.totext(c1, only_printable=True).strip().strip(':').strip()
         val = self.totext(c2)
         if not val:
             return
@@ -908,6 +909,10 @@ class Worker(Thread):  # Get details {{{
             ans = check_isbn(val)
             if ans:
                 self.isbn = mi.isbn = ans
+        elif name in {'Publication date'}:
+            from calibre.utils.date import parse_only_date
+            date = self.delocalize_datestr(val)
+            mi.pubdate = parse_only_date(date, assume_utc=True)
 
     def parse_isbn(self, pd):
         items = pd.xpath(
@@ -963,7 +968,7 @@ class Worker(Thread):  # Get details {{{
 class Amazon(Source):
 
     name = 'Amazon.com'
-    version = (1, 2, 18)
+    version = (1, 2, 21)
     minimum_calibre_version = (2, 82, 0)
     description = _('Downloads metadata and covers from Amazon')
 
@@ -1642,7 +1647,7 @@ def manual_tests(domain, **kw):  # {{{
 
         (  # No specific problems
             {'identifiers': {'isbn': '0743273567'}},
-            [title_test('The great gatsby', exact=True),
+            [title_test('the great gatsby: the only authorized edition', exact=True),
              authors_test(['Francis Scott Fitzgerald'])]
         ),
 
