@@ -10,9 +10,9 @@ import json
 import os
 import time
 from functools import partial
-from PyQt5.Qt import QAction, QIcon, Qt, pyqtSignal
+from qt.core import QAction, QIcon, Qt, pyqtSignal, QDialog
 
-from calibre.constants import ismacos, iswindows, plugins
+from calibre.constants import ismacos, iswindows
 from calibre.gui2 import (
     Dispatcher, config, elided_text, error_dialog, info_dialog, open_local_file,
     question_dialog
@@ -21,7 +21,17 @@ from calibre.gui2.actions import InterfaceAction
 from calibre.gui2.dialogs.choose_format import ChooseFormatDialog
 from calibre.ptempfile import PersistentTemporaryFile
 from calibre.utils.config import prefs, tweaks
-from polyglot.builtins import as_bytes, unicode_type
+from polyglot.builtins import as_bytes
+
+
+def preferred_format(formats):
+    formats = tuple(x.upper() for x in formats if x)
+    fmt = formats[0]
+    for format in prefs['input_format_order']:
+        if format in formats:
+            fmt = format
+            break
+    return fmt
 
 
 class HistoryAction(QAction):
@@ -63,6 +73,10 @@ class ViewAction(InterfaceAction):
         self.clear_history_action = cm('clear history',
                 _('Clear recently viewed list'), triggered=self.clear_history)
         self.history_actions = [self.clear_sep1]
+        self.action_view_last_read = ac = self.create_action(
+            spec=(_('Continue reading previous book'), None, _('Continue reading the last opened book'), 'shift+v'), attr='action_view_last_read')
+        ac.triggered.connect(self.view_last_read)
+        self.gui.addAction(ac)
 
     def initialization_complete(self):
         self.build_menus(self.gui.current_db)
@@ -82,6 +96,12 @@ class ViewAction(InterfaceAction):
                 ac.view_historical.connect(self.view_historical)
                 self.history_actions.append(ac)
 
+    def view_last_read(self):
+        history = self.gui.current_db.new_api.pref('gui_view_history', [])
+        for book_id, title in history:
+            self.view_historical(book_id)
+            break
+
     def browse_annots(self):
         self.gui.iactions['Browse Annotations'].show_browser()
 
@@ -118,7 +138,7 @@ class ViewAction(InterfaceAction):
                 merge_annotations(other_annotations_map, annotations_map, merge_last_read=False)
         return {
             'book_id': book_id, 'uuid': db.field_for('uuid', book_id), 'fmt': fmt.upper(),
-            'annotations_map': annotations_map,
+            'annotations_map': annotations_map, 'library_id': getattr(self.gui.current_db.new_api, 'server_library_id', None)
         }
 
     def view_format_by_id(self, id_, format, open_at=None):
@@ -137,7 +157,7 @@ class ViewAction(InterfaceAction):
         self._view_file(job.result)
 
     def _launch_viewer(self, name=None, viewer='ebook-viewer', internal=True, calibre_book_data=None, open_at=None):
-        self.gui.setCursor(Qt.BusyCursor)
+        self.gui.setCursor(Qt.CursorShape.BusyCursor)
         try:
             if internal:
                 args = [viewer]
@@ -156,11 +176,11 @@ class ViewAction(InterfaceAction):
                         kwargs=dict(args=args))
             else:
                 if iswindows:
-                    winutil = plugins['winutil'][0]
+                    from calibre_extensions import winutil
                     ext = name.rpartition('.')[-1]
                     if ext:
                         try:
-                            prog = winutil.file_association(unicode_type('.' + ext))
+                            prog = winutil.file_association(str('.' + ext))
                         except Exception:
                             prog = None
                         if prog and prog.lower().endswith('calibre.exe'):
@@ -208,7 +228,7 @@ class ViewAction(InterfaceAction):
         d = ChooseFormatDialog(self.gui, _('Choose the format to view'),
                 list(sorted(all_fmts)), show_open_with=True)
         self.gui.book_converted.connect(d.book_converted)
-        if d.exec_() == d.Accepted:
+        if d.exec_() == QDialog.DialogCode.Accepted:
             formats = [[x.upper() for x in db.new_api.formats(book_id)] for book_id in book_ids]
             fmt = d.format()
             orig_num = len(rows)
@@ -286,26 +306,20 @@ class ViewAction(InterfaceAction):
         views = []
         for id_ in ids:
             try:
-                formats = db.formats(id_, index_is_id=True)
+                title = db.title(id_, index_is_id=True)
             except:
                 error_dialog(self.gui, _('Cannot view'),
                     _('This book no longer exists in your library'), show=True)
                 self.update_history([], remove={id_})
                 continue
 
-            title   = db.title(id_, index_is_id=True)
+            formats = db.new_api.formats(id_, verify_formats=True)
             if not formats:
                 error_dialog(self.gui, _('Cannot view'),
                     _('%s has no available formats.')%(title,), show=True)
                 continue
 
-            formats = formats.upper().split(',')
-
-            fmt = formats[0]
-            for format in prefs['input_format_order']:
-                if format in formats:
-                    fmt = format
-                    break
+            fmt = preferred_format(formats)
             views.append((id_, title))
             self.view_format_by_id(id_, fmt)
 
