@@ -30,7 +30,7 @@ from calibre.gui2.library import DEFAULT_SORT
 from calibre.constants import filesystem_encoding
 from calibre import force_unicode
 from calibre.utils.icu import primary_sort_key
-from polyglot.builtins import iteritems, map, range, unicode_type
+from polyglot.builtins import iteritems
 
 
 def restrict_column_width(self, col, old_size, new_size):
@@ -74,7 +74,7 @@ class HeaderView(QHeaderView):  # {{{
         opt.orientation = self.orientation()
         opt.fontMetrics = self.fm
         model = self.parent().model()
-        opt.text = unicode_type(model.headerData(logical_index, opt.orientation, Qt.ItemDataRole.DisplayRole) or '')
+        opt.text = str(model.headerData(logical_index, opt.orientation, Qt.ItemDataRole.DisplayRole) or '')
         if opt.orientation == Qt.Orientation.Vertical:
             try:
                 val = model.headerData(logical_index, opt.orientation, Qt.ItemDataRole.DecorationRole)
@@ -102,7 +102,7 @@ class HeaderView(QHeaderView):  # {{{
             opt.sortIndicator = QStyleOptionHeader.SortIndicator.SortDown if \
                 self.sortIndicatorOrder() == Qt.SortOrder.AscendingOrder else QStyleOptionHeader.SortIndicator.SortUp
             margin += style.pixelMetric(QStyle.PixelMetric.PM_HeaderMarkSize, None, self)
-        opt.text = unicode_type(model.headerData(logical_index, opt.orientation, Qt.ItemDataRole.DisplayRole) or '')
+        opt.text = str(model.headerData(logical_index, opt.orientation, Qt.ItemDataRole.DisplayRole) or '')
         if self.textElideMode() != Qt.TextElideMode.ElideNone:
             opt.text = opt.fontMetrics.elidedText(opt.text, Qt.TextElideMode.ElideRight, rect.width() - margin)
         if self.isEnabled():
@@ -133,7 +133,7 @@ class HeaderView(QHeaderView):  # {{{
 # }}}
 
 
-class PreserveViewState(object):  # {{{
+class PreserveViewState:  # {{{
 
     '''
     Save the set of selected books at enter time. If at exit time there are no
@@ -237,6 +237,8 @@ class BooksView(QTableView):  # {{{
         for wv in self, self.pin_view:
             if not tweaks['horizontal_scrolling_per_column']:
                 wv.setHorizontalScrollMode(QAbstractItemView.ScrollMode.ScrollPerPixel)
+            if not tweaks['vertical_scrolling_per_row']:
+                wv.setVerticalScrollMode(QAbstractItemView.ScrollMode.ScrollPerPixel)
 
             wv.setEditTriggers(QAbstractItemView.EditTrigger.EditKeyPressed)
             tval = tweaks['doubleclick_on_library_view']
@@ -478,7 +480,7 @@ class BooksView(QTableView):  # {{{
         ans.addSeparator()
         if hidden_cols:
             m = ans.addMenu(_('Show column'))
-            hcols = [(hcol, unicode_type(self.model().headerData(hidx, Qt.Orientation.Horizontal, Qt.ItemDataRole.DisplayRole) or ''))
+            hcols = [(hcol, str(self.model().headerData(hidx, Qt.Orientation.Horizontal, Qt.ItemDataRole.DisplayRole) or ''))
                      for hcol, hidx in iteritems(hidden_cols)]
             hcols.sort(key=lambda x: primary_sort_key(x[1]))
             for hcol, hname in hcols:
@@ -514,7 +516,7 @@ class BooksView(QTableView):  # {{{
         col = None
         if idx > -1 and idx < len(self.column_map):
             col = self.column_map[idx]
-            name = unicode_type(self.model().headerData(idx, Qt.Orientation.Horizontal, Qt.ItemDataRole.DisplayRole) or '')
+            name = str(self.model().headerData(idx, Qt.Orientation.Horizontal, Qt.ItemDataRole.DisplayRole) or '')
             view.column_header_context_menu = self.create_context_menu(col, name, view)
         has_context_menu = hasattr(view, 'column_header_context_menu')
         if self.is_library_view and has_context_menu:
@@ -672,7 +674,7 @@ class BooksView(QTableView):  # {{{
 
     def write_state(self, state):
         db = getattr(self.model(), 'db', None)
-        name = unicode_type(self.objectName())
+        name = str(self.objectName())
         if name and db is not None:
             db.new_api.set_pref(name + ' books view state', state)
 
@@ -787,7 +789,7 @@ class BooksView(QTableView):  # {{{
 
     def get_old_state(self):
         ans = None
-        name = unicode_type(self.objectName())
+        name = str(self.objectName())
         if name:
             name += ' books view state'
             db = getattr(self.model(), 'db', None)
@@ -988,7 +990,7 @@ class BooksView(QTableView):  # {{{
 
         self.restore_state()
         self.set_ondevice_column_visibility()
-        # incase there were marked books
+        # in case there were marked books
         self.model().set_row_decoration(set())
         self.row_header.headerDataChanged(Qt.Orientation.Vertical, 0, self.row_header.count()-1)
         self.row_header.geometriesChanged.emit()
@@ -1205,7 +1207,7 @@ class BooksView(QTableView):  # {{{
     def selectionCommand(self, index, event):
         if event and event.type() == QEvent.Type.KeyPress and event.key() in (Qt.Key.Key_Home, Qt.Key.Key_End) and event.modifiers() & Qt.Modifier.CTRL:
             return QItemSelectionModel.SelectionFlag.ClearAndSelect | QItemSelectionModel.SelectionFlag.Rows
-        return super(BooksView, self).selectionCommand(index, event)
+        return super().selectionCommand(index, event)
 
     def keyPressEvent(self, ev):
         if handle_enter_press(self, ev):
@@ -1307,8 +1309,8 @@ class BooksView(QTableView):  # {{{
         ci = self.currentIndex()
         if not ci.isValid():
             return None
-        selected_rows = frozenset([i.row() for i in self.selectedIndexes() if
-            i.isValid()])
+        selected_rows = frozenset(i.row() for i in self.selectedIndexes() if
+            i.isValid())
         column = ci.column()
 
         for i in range(ci.row()+1, self.row_count()):
@@ -1426,14 +1428,14 @@ class DeviceBooksView(BooksView):  # {{{
 
     def get_old_state(self):
         ans = None
-        name = unicode_type(self.objectName())
+        name = str(self.objectName())
         if name:
             name += ' books view state'
             ans = gprefs.get(name, None)
         return ans
 
     def write_state(self, state):
-        name = unicode_type(self.objectName())
+        name = str(self.objectName())
         if name:
             gprefs.set(name + ' books view state', state)
 
