@@ -17,7 +17,6 @@ from calibre.ebooks.oeb.polish.check.main import run_checks, fix_errors
 from calibre.gui2 import NO_URL_FORMATTING, safe_open_url
 from calibre.gui2.tweak_book import tprefs
 from calibre.gui2.tweak_book.widgets import BusyCursor
-from polyglot.builtins import unicode_type, range
 
 
 def icon_for_level(level):
@@ -49,7 +48,7 @@ def prefix_for_level(level):
 class Delegate(QStyledItemDelegate):
 
     def initStyleOption(self, option, index):
-        super(Delegate, self).initStyleOption(option, index)
+        super().initStyleOption(option, index)
         if index.row() == self.parent().currentRow():
             option.font.setBold(True)
             option.backgroundBrush = self.parent().palette().brush(QPalette.ColorRole.AlternateBase)
@@ -101,7 +100,7 @@ class Check(QSplitter):
     def copy_to_clipboard(self):
         items = []
         for item in (self.items.item(i) for i in range(self.items.count())):
-            msg = unicode_type(item.text())
+            msg = str(item.text())
             msg = prefix_for_level(item.data(Qt.ItemDataRole.UserRole).level) + msg
             items.append(msg)
         if items:
@@ -117,7 +116,7 @@ class Check(QSplitter):
             msg, _('Click to run a check on the book'), _('Run check')))
 
     def link_clicked(self, url):
-        url = unicode_type(url.toString(NO_URL_FORMATTING))
+        url = str(url.toString(NO_URL_FORMATTING))
         if url == 'activate:item':
             self.current_item_activated()
         elif url == 'run:check':
@@ -245,7 +244,7 @@ class Check(QSplitter):
     def keyPressEvent(self, ev):
         if ev.key() in (Qt.Key.Key_Enter, Qt.Key.Key_Return):
             self.current_item_activated()
-        return super(Check, self).keyPressEvent(ev)
+        return super().keyPressEvent(ev)
 
     def clear(self):
         self.items.clear()
