@@ -60,7 +60,7 @@ from calibre.library import current_library_name
 from calibre.srv.library_broker import GuiLibraryBroker, db_matches
 from calibre.utils.config import dynamic, prefs
 from calibre.utils.ipc.pool import Pool
-from polyglot.builtins import string_or_bytes, unicode_type
+from polyglot.builtins import string_or_bytes
 from polyglot.queue import Empty, Queue
 
 
@@ -80,7 +80,7 @@ def add_quick_start_guide(library_view, refresh_cover_browser=None):
     try:
         with lopen(P('quick_start/%s.epub' % l), 'rb') as src:
             buf = BytesIO(src.read())
-    except EnvironmentError as err:
+    except OSError as err:
         if err.errno != errno.ENOENT:
             raise
         with lopen(P('quick_start/eng.epub'), 'rb') as src:
@@ -109,6 +109,8 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
 
     proceed_requested = pyqtSignal(object, object)
     book_converted = pyqtSignal(object, object)
+    enter_key_pressed_in_book_list = pyqtSignal(object)  # used by action chains plugin
+    event_in_db = pyqtSignal(object, object, object)  # (db, event_type, event_data)
     shutting_down = False
 
     def __init__(self, opts, parent=None, gui_debug=None):
@@ -199,6 +201,13 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         else:
             stmap[st.name] = st
 
+    def add_db_listener(self, callback):
+        self.library_broker.start_listening_for_db_events()
+        self.event_in_db.connect(callback)
+
+    def remove_db_listener(self, callback):
+        self.event_in_db.disconnect(callback)
+
     def initialize(self, library_path, db, actions, show_gui=True):
         opts = self.opts
         self.preferences_action, self.quit_action = actions
@@ -484,7 +493,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
             if os.path.exists(os.path.join(config_dir, 'server.py')):
                 try:
                     os.remove(os.path.join(config_dir, 'server.py'))
-                except EnvironmentError:
+                except OSError:
                     pass
                 warning_dialog(self, _('Content server changed!'), _(
                     'calibre 3 comes with a completely re-written Content server.'
@@ -597,7 +606,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         if self.content_server is not None and \
                 self.content_server.exception is not None:
             error_dialog(self, _('Failed to start Content server'),
-                         unicode_type(self.content_server.exception)).exec_()
+                         str(self.content_server.exception)).exec_()
 
     @property
     def current_db(self):
@@ -784,6 +793,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
                     db.format_metadata(book_id, fmt, allow_cache=False, update_db=True)
                     db.update_last_modified((book_id,))
                     m.refresh_ids((book_id,))
+                    db.event_dispatcher(db.EventType.book_edited, book_id, fmt)
             except Exception:
                 import traceback
                 traceback.print_exc()
@@ -910,7 +920,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
             font.setBold(True)
             font.setItalic(True)
         self.virtual_library.setFont(font)
-        title = '{0} — || {1}{2} ||'.format(
+        title = '{} — || {}{} ||'.format(
                 __appname__, self.iactions['Choose Library'].library_name(), restrictions)
         self.setWindowTitle(title)
 
@@ -1033,7 +1043,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
             pass
         if not minz:
             self.job_error_dialog.show_error(dialog_title,
-                    _('<b>Failed</b>')+': '+unicode_type(job.description),
+                    _('<b>Failed</b>')+': '+str(job.description),
                     det_msg=job.details, retry_func=retry_func)
 
     def read_settings(self):
@@ -1050,7 +1060,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
             self.stack.tb_widget.save_state()
 
     def quit(self, checked=True, restart=False, debug_on_restart=False,
-            confirm_quit=True):
+            confirm_quit=True, no_plugins_on_restart=False):
         if self.shutting_down:
             return
         if confirm_quit and not self.confirm_quit():
@@ -1062,6 +1072,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
             traceback.print_exc()
         self.restart_after_quit = restart
         self.debug_on_restart = debug_on_restart
+        self.no_plugins_on_restart = no_plugins_on_restart
         if self.system_tray_icon is not None and self.restart_after_quit:
             # Needed on windows to prevent multiple systray icons
             self.system_tray_icon.setVisible(False)
@@ -1102,6 +1113,10 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, EmailMixin,  # {{{
         self.shutting_down = True
         self.show_shutdown_message()
         self.server_change_notification_timer.stop()
+        try:
+            self.event_in_db.disconnect()
+        except Exception:
+            pass
 
         from calibre.customize.ui import has_library_closed_plugins
         if has_library_closed_plugins():
