@@ -6,11 +6,12 @@
 import os
 from functools import partial
 
-from PyQt5.Qt import (
+from qt.core import (
     QAction, QGroupBox, QHBoxLayout, QIcon, QKeySequence, QLabel, QListWidget,
-    QListWidgetItem, QMenu, Qt, QToolBar, QToolButton, QVBoxLayout, pyqtSignal
+    QListWidgetItem, QMenu, Qt, QToolBar, QToolButton, QVBoxLayout, pyqtSignal, QDialog,
+    QAbstractItemView, QDialogButtonBox
 )
-from PyQt5.QtWebEngineWidgets import QWebEnginePage
+from qt.webengine import QWebEnginePage
 
 from calibre.constants import ismacos
 from calibre.gui2 import elided_text
@@ -19,10 +20,9 @@ from calibre.gui2.viewer.shortcuts import index_to_key_sequence
 from calibre.gui2.viewer.web_view import set_book_path, vprefs
 from calibre.gui2.widgets2 import Dialog
 from calibre.utils.icu import primary_sort_key
-from polyglot.builtins import iteritems
 
 
-class Action(object):
+class Action:
 
     __slots__ = ('icon', 'text', 'shortcut_action')
 
@@ -30,7 +30,7 @@ class Action(object):
         self.icon, self.text, self.shortcut_action = QIcon(I(icon)), text, shortcut_action
 
 
-class Actions(object):
+class Actions:
 
     def __init__(self, a):
         self.__dict__.update(a)
@@ -44,7 +44,7 @@ def all_actions():
             'back': Action('back.png', _('Back')),
             'forward': Action('forward.png', _('Forward')),
             'open': Action('document_open.png', _('Open e-book')),
-            'copy': Action('edit-copy.png', _('Copy to clipboard')),
+            'copy': Action('edit-copy.png', _('Copy to clipboard'), 'copy_to_clipboard'),
             'increase_font_size': Action('font_size_larger.png', _('Increase font size'), 'increase_font_size'),
             'decrease_font_size': Action('font_size_smaller.png', _('Decrease font size'), 'decrease_font_size'),
             'fullscreen': Action('page.png', _('Toggle full screen'), 'toggle_full_screen'),
@@ -64,8 +64,11 @@ def all_actions():
             'print': Action('print.png', _('Print book'), 'print'),
             'preferences': Action('config.png', _('Preferences'), 'preferences'),
             'metadata': Action('metadata.png', _('Show book metadata'), 'metadata'),
-            'highlight': Action('highlight.png', _('Highlight text in the book'), 'create_annotation'),
+            'toggle_read_aloud': Action('bullhorn.png', _('Read aloud'), 'toggle_read_aloud'),
             'toggle_highlights': Action('highlight_only_on.png', _('Browse highlights in book'), 'toggle_highlights'),
+            'select_all': Action('edit-select-all.png', _('Select all text in the current file')),
+            'edit_book': Action('edit_book.png', _('Edit this book'), 'edit_book'),
+            'reload_book': Action('reload.png', _('Reload this book'), 'reload_book'),
         }
         all_actions.ans = Actions(amap)
     return all_actions.ans
@@ -73,7 +76,7 @@ def all_actions():
 
 DEFAULT_ACTIONS = (
     'back', 'forward', None, 'open', 'copy', 'increase_font_size', 'decrease_font_size', 'fullscreen', 'color_scheme',
-    None, 'previous', 'next', None, 'toc', 'search', 'bookmarks', 'lookup', 'highlight', 'chrome', None,
+    None, 'previous', 'next', None, 'toc', 'search', 'bookmarks', 'lookup', 'toggle_highlights', 'chrome', None,
     'mode', 'print', 'preferences', 'metadata', 'inspector'
 )
 
@@ -91,10 +94,10 @@ class ToolBar(QToolBar):
         QToolBar.__init__(self, parent)
         self.setWindowTitle(_('Toolbar'))
         self.shortcut_actions = {}
-        self.setToolButtonStyle(Qt.ToolButtonIconOnly)
+        self.setToolButtonStyle(Qt.ToolButtonStyle.ToolButtonIconOnly)
         self.setVisible(False)
-        self.setAllowedAreas(Qt.AllToolBarAreas)
-        self.setContextMenuPolicy(Qt.CustomContextMenu)
+        self.setAllowedAreas(Qt.ToolBarArea.AllToolBarAreas)
+        self.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
 
     def create_shortcut_action(self, name):
         a = getattr(all_actions(), name)
@@ -113,15 +116,22 @@ class ActionsToolBar(ToolBar):
     def __init__(self, parent=None):
         ToolBar.__init__(self, parent)
         self.setObjectName('actions_toolbar')
+        self.prevent_sleep_cookie = None
         self.customContextMenuRequested.connect(self.show_context_menu)
 
+    def update_action_state(self, book_open):
+        for ac in self.shortcut_actions.values():
+            ac.setEnabled(book_open)
+        self.search_action.setEnabled(book_open)
+        self.color_scheme_action.setEnabled(book_open)
+
     def show_context_menu(self, pos):
         m = QMenu(self)
         a = m.addAction(_('Customize this toolbar'))
         a.triggered.connect(self.customize)
         a = m.addAction(_('Hide this toolbar'))
         a.triggered.connect(self.hide_toolbar)
-        m.exec_(pos)
+        m.exec_(self.mapToGlobal(pos))
 
     def hide_toolbar(self):
         self.web_view.trigger_shortcut('toggle_toolbar')
@@ -136,23 +146,25 @@ class ActionsToolBar(ToolBar):
         web_view.reference_mode_changed.connect(self.update_reference_mode_action)
         web_view.standalone_misc_settings_changed.connect(self.update_visibility)
         web_view.autoscroll_state_changed.connect(self.update_autoscroll_action)
-        web_view.customize_toolbar.connect(self.customize, type=Qt.QueuedConnection)
+        web_view.read_aloud_state_changed.connect(self.update_read_aloud_action)
+        web_view.customize_toolbar.connect(self.customize, type=Qt.ConnectionType.QueuedConnection)
         web_view.view_created.connect(self.on_view_created)
 
-        self.back_action = page.action(QWebEnginePage.Back)
+        self.back_action = page.action(QWebEnginePage.WebAction.Back)
         self.back_action.setIcon(aa.back.icon)
         self.back_action.setText(aa.back.text)
-        self.forward_action = page.action(QWebEnginePage.Forward)
+        self.forward_action = page.action(QWebEnginePage.WebAction.Forward)
         self.forward_action.setIcon(aa.forward.icon)
         self.forward_action.setText(aa.forward.text)
+        self.select_all_action = a = page.action(QWebEnginePage.WebAction.SelectAll)
+        a.setIcon(aa.select_all.icon), a.setText(aa.select_all.text)
 
         self.open_action = a = QAction(aa.open.icon, aa.open.text, self)
         self.open_menu = m = QMenu(self)
         a.setMenu(m)
         m.aboutToShow.connect(self.populate_open_menu)
         connect_lambda(a.triggered, self, lambda self: self.open_book_at_path.emit(None))
-        self.copy_action = a = page.action(QWebEnginePage.Copy)
-        a.setIcon(aa.copy.icon), a.setText(aa.copy.text)
+        self.copy_action = shortcut_action('copy')
         self.increase_font_size_action = shortcut_action('increase_font_size')
         self.decrease_font_size_action = shortcut_action('decrease_font_size')
         self.fullscreen_action = shortcut_action('fullscreen')
@@ -170,9 +182,10 @@ class ActionsToolBar(ToolBar):
         a.setCheckable(True)
         self.reference_action = a = shortcut_action('reference')
         a.setCheckable(True)
-        self.highlight_action = a = shortcut_action('highlight')
         self.toggle_highlights_action = self.highlights_action = a = shortcut_action('toggle_highlights')
         a.setCheckable(True)
+        self.toggle_read_aloud_action = a = shortcut_action('toggle_read_aloud')
+        a.setCheckable(True)
         self.lookup_action = a = shortcut_action('lookup')
         a.setCheckable(True)
         self.inspector_action = a = shortcut_action('inspector')
@@ -180,6 +193,7 @@ class ActionsToolBar(ToolBar):
         self.autoscroll_action = a = shortcut_action('autoscroll')
         a.setCheckable(True)
         self.update_autoscroll_action(False)
+        self.update_read_aloud_action(False)
         self.chrome_action = shortcut_action('chrome')
 
         self.mode_action = a = shortcut_action('mode')
@@ -187,6 +201,8 @@ class ActionsToolBar(ToolBar):
         self.print_action = shortcut_action('print')
         self.preferences_action = shortcut_action('preferences')
         self.metadata_action = shortcut_action('metadata')
+        self.edit_book_action = shortcut_action('edit_book')
+        self.reload_book_action = shortcut_action('reload_book')
         self.update_mode_action()
         self.color_scheme_action = a = QAction(aa.color_scheme.icon, aa.color_scheme.text, self)
         self.color_scheme_menu = m = QMenu(self)
@@ -208,7 +224,7 @@ class ActionsToolBar(ToolBar):
                     pass
         w = self.widgetForAction(self.color_scheme_action)
         if w:
-            w.setPopupMode(w.InstantPopup)
+            w.setPopupMode(QToolButton.ToolButtonPopupMode.InstantPopup)
 
     def update_mode_action(self):
         mode = get_session_pref('read_mode', default='paged', group=None)
@@ -220,10 +236,35 @@ class ActionsToolBar(ToolBar):
             a.setChecked(True)
             a.setToolTip(_('Switch to paged mode -- where the text is broken into pages'))
 
+    def change_sleep_permission(self, disallow_sleep=True):
+        from .control_sleep import prevent_sleep, allow_sleep
+        if disallow_sleep:
+            if self.prevent_sleep_cookie is None:
+                try:
+                    self.prevent_sleep_cookie = prevent_sleep()
+                except Exception:
+                    import traceback
+                    traceback.print_exc()
+        else:
+            if self.prevent_sleep_cookie is not None:
+                try:
+                    allow_sleep(self.prevent_sleep_cookie)
+                except Exception:
+                    import traceback
+                    traceback.print_exc()
+                self.prevent_sleep_cookie = None
+
     def update_autoscroll_action(self, active):
         self.autoscroll_action.setChecked(active)
         self.autoscroll_action.setToolTip(
             _('Turn off auto-scrolling') if active else _('Turn on auto-scrolling'))
+        self.change_sleep_permission(active)
+
+    def update_read_aloud_action(self, active):
+        self.toggle_read_aloud_action.setChecked(active)
+        self.toggle_read_aloud_action.setToolTip(
+            _('Stop reading') if active else _('Read the text of the book aloud'))
+        self.change_sleep_permission(active)
 
     def update_reference_mode_action(self, enabled):
         self.reference_action.setChecked(enabled)
@@ -234,19 +275,25 @@ class ActionsToolBar(ToolBar):
             ac.setChecked(visibility_map[k])
 
     def set_tooltips(self, rmap):
-        for sc, a in iteritems(self.shortcut_actions):
-            if a.isCheckable():
-                continue
+        aliases = {'show_chrome_force': 'show_chrome'}
+
+        def as_text(idx):
+            return index_to_key_sequence(idx).toString(QKeySequence.SequenceFormat.NativeText)
+
+        def set_it(a, sc):
             x = rmap.get(sc)
             if x is not None:
-
-                def as_text(idx):
-                    return index_to_key_sequence(idx).toString(QKeySequence.NativeText)
-
                 keys = sorted(filter(None, map(as_text, x)))
                 if keys:
                     a.setToolTip('{} [{}]'.format(a.text(), ', '.join(keys)))
 
+        for sc, a in self.shortcut_actions.items():
+            sc = aliases.get(sc, sc)
+            set_it(a, sc)
+
+        for a, sc in ((self.forward_action, 'forward'), (self.back_action, 'back'), (self.search_action, 'start_search')):
+            set_it(a, sc)
+
     def populate_open_menu(self):
         m = self.open_menu
         m.clear()
@@ -300,7 +347,7 @@ class ActionsToolBar(ToolBar):
 
     def customize(self):
         d = ConfigureToolBar(parent=self.parent())
-        if d.exec_() == d.Accepted:
+        if d.exec_() == QDialog.DialogCode.Accepted:
             self.add_actions()
 
 
@@ -308,12 +355,12 @@ class ActionsList(QListWidget):
 
     def __init__(self, actions, parent=None, is_source=True):
         QListWidget.__init__(self, parent)
-        self.setSelectionMode(self.ExtendedSelection)
+        self.setSelectionMode(QAbstractItemView.SelectionMode.ExtendedSelection)
         self.setDragEnabled(True)
         self.viewport().setAcceptDrops(True)
         self.setDropIndicatorShown(True)
-        self.setDragDropMode(self.InternalMove)
-        self.setDefaultDropAction(Qt.CopyAction if ismacos else Qt.MoveAction)
+        self.setDragDropMode(QAbstractItemView.DragDropMode.InternalMove)
+        self.setDefaultDropAction(Qt.DropAction.CopyAction if ismacos else Qt.DropAction.MoveAction)
         self.setMinimumHeight(400)
         self.is_source = is_source
         if is_source:
@@ -335,7 +382,7 @@ class ActionsList(QListWidget):
             except AttributeError:
                 return
             i = QListWidgetItem(a.icon, a.text, self)
-        i.setData(Qt.UserRole, action)
+        i.setData(Qt.ItemDataRole.UserRole, action)
         return i
 
     def set_names(self, names):
@@ -343,13 +390,16 @@ class ActionsList(QListWidget):
         for name in names:
             self.add_item_from_name(name)
 
+    def remove_item(self, item):
+        action = item.data(Qt.ItemDataRole.UserRole)
+        if action is not None or not self.is_source:
+            self.takeItem(self.row(item))
+        return action
+
     def remove_selected(self):
         ans = []
         for item in tuple(self.selectedItems()):
-            action = item.data(Qt.UserRole)
-            if action is not None or not self.is_source:
-                self.takeItem(self.row(item))
-            ans.append(action)
+            ans.append(self.remove_item(item))
         return ans
 
     def add_names(self, names):
@@ -364,7 +414,7 @@ class ActionsList(QListWidget):
     def names(self):
         for i in range(self.count()):
             item = self.item(i)
-            yield item.data(Qt.UserRole)
+            yield item.data(Qt.ItemDataRole.UserRole)
 
 
 class ConfigureToolBar(Dialog):
@@ -375,7 +425,9 @@ class ConfigureToolBar(Dialog):
     def setup_ui(self):
         acnames = all_actions().all_action_names
         self.available_actions = ActionsList(acnames - frozenset(current_actions()), parent=self)
+        self.available_actions.itemDoubleClicked.connect(self.add_item)
         self.current_actions = ActionsList(current_actions(), parent=self, is_source=False)
+        self.current_actions.itemDoubleClicked.connect(self.remove_item)
         self.l = l = QVBoxLayout(self)
         self.la = la = QLabel(_('Choose the actions you want on the toolbar.'
             ' Drag and drop items in the right hand list to re-arrange the toolbar.'))
@@ -403,17 +455,25 @@ class ConfigureToolBar(Dialog):
         v.addWidget(self.current_actions)
         h.addLayout(bv), h.addWidget(rg)
         l.addWidget(self.bb)
-        self.rdb = b = self.bb.addButton(_('Restore defaults'), self.bb.ActionRole)
+        self.rdb = b = self.bb.addButton(_('Restore defaults'), QDialogButtonBox.ButtonRole.ActionRole)
         b.clicked.connect(self.restore_defaults)
 
     def remove_actions(self):
         names = self.current_actions.remove_selected()
         self.available_actions.add_names(names)
 
+    def remove_item(self, item):
+        names = self.current_actions.remove_item(item),
+        self.available_actions.add_names(names)
+
     def add_actions(self):
         names = self.available_actions.remove_selected()
         self.current_actions.add_names(names)
 
+    def add_item(self, item):
+        names = self.available_actions.remove_item(item),
+        self.current_actions.add_names(names)
+
     def restore_defaults(self):
         self.current_actions.set_names(DEFAULT_ACTIONS)
         acnames = all_actions().all_action_names
