@@ -13,7 +13,7 @@ Test a binary calibre build to ensure that all needed binary images/libraries ha
 import os, ctypes, sys, unittest, time, shutil
 
 from calibre.constants import iswindows, islinux, ismacos, plugins_loc
-from polyglot.builtins import iteritems, map, unicode_type, getenv
+from polyglot.builtins import iteritems
 
 is_ci = os.environ.get('CI', '').lower() == 'true'
 is_sanitized = 'libasan' in os.environ.get('LD_PRELOAD', '')
@@ -38,15 +38,12 @@ class BuildTest(unittest.TestCase):
 
     @unittest.skipUnless(islinux, 'DBUS only used on linux')
     def test_dbus(self):
-        try:
-            import dbus
-        except SyntaxError:
-            return  # an invalid \ in a string causes a syntax error, but only in CI
+        from jeepney.io.blocking import open_dbus_connection
         if 'DBUS_SESSION_BUS_ADDRESS' in os.environ:
-            bus = dbus.SystemBus()
-            self.assertTrue(bus.list_names(), 'Failed to list names on the system bus')
-            bus = dbus.SessionBus()
-            self.assertTrue(bus.list_names(), 'Failed to list names on the session bus')
+            bus = open_dbus_connection(bus='SYSTEM')
+            bus.close()
+            bus = open_dbus_connection(bus='SESSION')
+            bus.close()
             del bus
 
     def test_loaders(self):
@@ -74,7 +71,7 @@ class BuildTest(unittest.TestCase):
 
     def test_chardet(self):
         from cchardet import detect
-        raw = 'mūsi Füße'.encode('utf-8')
+        raw = 'mūsi Füße'.encode()
         data = detect(raw)
         self.assertEqual(data['encoding'].lower(), 'utf-8')
         self.assertGreater(data['confidence'], 0.5)
@@ -109,6 +106,8 @@ class BuildTest(unittest.TestCase):
 
     def test_zeroconf(self):
         import zeroconf as z, ifaddr
+        from calibre.devices.smart_device_app.driver import monkeypatch_zeroconf
+        monkeypatch_zeroconf()
         del z
         del ifaddr
 
@@ -148,7 +147,7 @@ class BuildTest(unittest.TestCase):
             s = msgpack_dumps(obj)
             self.assertEqual(obj, msgpack_loads(s))
         self.assertEqual(type(msgpack_loads(msgpack_dumps(b'b'))), bytes)
-        self.assertEqual(type(msgpack_loads(msgpack_dumps('b'))), unicode_type)
+        self.assertEqual(type(msgpack_loads(msgpack_dumps('b'))), str)
         large = b'x' * (100 * 1024 * 1024)
         msgpack_loads(msgpack_dumps(large))
 
@@ -169,7 +168,7 @@ class BuildTest(unittest.TestCase):
 
         def au(x, name):
             self.assertTrue(
-                isinstance(x, unicode_type),
+                isinstance(x, str),
                 '%s() did not return a unicode string, instead returning: %r' % (name, x))
         for x in 'username temp_path locale_name'.split():
             au(getattr(winutil, x)(), x)
@@ -179,11 +178,11 @@ class BuildTest(unittest.TestCase):
         for k, v in iteritems(d):
             au(v, k)
         os.environ['XXXTEST'] = 'YYY'
-        self.assertEqual(getenv('XXXTEST'), 'YYY')
+        self.assertEqual(os.getenv('XXXTEST'), 'YYY')
         del os.environ['XXXTEST']
-        self.assertIsNone(getenv('XXXTEST'))
+        self.assertIsNone(os.getenv('XXXTEST'))
         for k in os.environ:
-            v = getenv(k)
+            v = os.getenv(k)
             if v is not None:
                 au(v, 'getenv-' + k)
         t = time.localtime()
@@ -347,6 +346,7 @@ class BuildTest(unittest.TestCase):
             QApplication.instance().exec_()
             test_flaky = ismacos and not is_ci
             if not test_flaky:
+                self.assertTrue(hasattr(callback, 'result'), 'Qt WebEngine failed to run in 5 seconds')
                 self.assertEqual(callback.result, 2, 'Simple JS computation failed')
                 self.assertIn(b'Skia/PDF', bytes(print_callback.result), 'Print to PDF failed')
             del p
